#!/usr/bin/env python
# coding: utf-8
import cmd
from keyboardHandler import KeyboardHandler
from __init__ import __version__
import sys
reload(sys)

sys.setdefaultencoding("utf-8")


class CLI(cmd.Cmd):

    def __init__(self):
        cmd.Cmd.__init__(self)
        self.intro = "Tickeys %s - Linux\nType 'help' for help" % __version__
        self.prompt = ">>> "
        self.detecter = KeyboardHandler()
        self.detecter.start_detecting()

        self.volume = self.detecter.get_player_infor()['volume'] * 100.0
        self.pitch = self.detecter.get_player_infor()['pitch'] * 10.0
        self.style = self.detecter.get_player_infor()['style']

    def default(self, line):
        print "Command '%s' is invalid, try 'help'" % line

    def help_setstyle(self):
        print "Set style, change the sound's effect"

    def do_setstyle(self, arg):
        style_index = raw_input(
            "Input the effect style number you want"
            "(0:bubble 1:mechanical 2:sword 3:typewriter 4:Cherry_G80_3000 5:Cherry_G80_3494 6:drum):")
        style_list = ['bubble', 'mechanical', 'sword', 'typewriter', 'Cherry_G80_3000', 'Cherry_G80_3494', 'drum']

        try:
            style_index = int(style_index)
            assert(0 <= style_index <= 6)
        except Exception:
            print "Input must between 0~6!!"
            return

        self.style = style_list[style_index]
        self.detecter.set_style(self.style)

    def help_setvol(self):
        print "Set volume, input the volume you want"

    def do_setvol(self, arg):
        volume = raw_input("Input the volume(0~100) you want:")

        try:
            volume = float(volume)
            assert(0 <= volume <= 100)
        except Exception:
            print "Volume must between 0~100!!"
            return

        self.volume = volume
        self.detecter.set_volume(self.volume/100.0)

    # def help_getvol(self):
    #     print "Get the volume"

    # def do_getvol(self, arg):
    #     print self.volume

    def help_setpitch(self):
        print "Set pitch, input the pitch you want"

    def do_setpitch(self, arg):
        pitch = raw_input("Input the pitch(0~30, default 10) you want:")

        try:
            pitch = float(pitch)
            assert(0 <= pitch <= 30)
        except Exception:
            print "Pitch must between 0~30!!"
            return

        self.pitch = pitch
        self.detecter.set_pitch(self.pitch/10.0)

    # def help_getpitch(self):
    #     print "Get the pitch"

    # def do_getpitch(self, arg):
    #     print self.pitch

    def help_getinfo(self):
        print "Get tickeys' sound effect, volume and pitch"

    def do_getinfo(self, arg):
        print "Sound effect: %s  Volume: %s  Pitch: %s" \
            % (self.style, self.volume, self.pitch)

    def do_quit(self, arg):
        try:
            self.detecter.stop_detecting()
        except Exception:
            pass
        finally:
            sys.exit(0)
            return True


if __name__ == "__main__":
    cli = CLI()
    cli.cmdloop()



#!/usr/bin/env python
# coding: utf-8
from logger import logger
import sys
import os

from windowManager import check_tickeys_running_status

__version__ = '0.2.2'
__author__ = 'Huang xiongbiao(billo@qq.com)'


def run_GUI():
    check_root()
    check_system()
    try:
        from GUI import TickeysApp
        TickeysApp().run()
    except Exception, e:
        logger.info("Run GUI Fail, use CLI instead..Fail msg:%s" % str(e))
        run_CLI()


def run_CLI():
    check_root()
    check_system()
    from CLI import CLI
    CLI().cmdloop()


def check_root():
    logger.info("Root checking...")
    if os.getegid() != 0:
        logger.info("This program must be run as root..")
        sys.exit(0)
    logger.info("Root checking success. You have the root permission")
    logger.debug("File path:" + os.path.dirname(__file__))

def check_system():
    systems = ['Linux', 'SunOS', 'FreeBSD', 'Unix', 'OpenBSD', 'NetBSD']
    try:
        logger.info("System checking...")
        import platform
        system_name = platform.system()
        if system_name not in systems:
            logger.error("System %s is not supported." % system_name)
            sys.exit(0)
        else:
            logger.info("System checking success. Your system is supported")
    except Exception, e:
        logger.error("System checking fail:" + str(e))
        sys.exit(0)


def print_help_msg():
    print "Tickeys will run GUI by default"
    print "Usage: -c     ---CLI mode"
    print "       -g     ---GUI mode"


def main():
    logger.debug("Tickeys start........")
    is_running = check_tickeys_running_status()
    if is_running:
        return
    if len(sys.argv) == 1:
        run_GUI()
    elif len(sys.argv) == 2:
        if sys.argv[1] == '-g':
            del sys.argv[1]  # otherwise kivy would regard it as option
            run_GUI()
        elif sys.argv[1] == '-c':
            run_CLI()
        else:
            print_help_msg()
    else:
        print_help_msg()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# coding:utf-8

from __future__ import with_statement
from logger import logger

import os
import commands

executable_filename = "tickeys"
python2 = os.popen('python2 -V 2>&1').read().startswith('Python 2.') and 'python2' or 'python'
DesktopEntryName = 'Tickeys.desktop'
StartupPath = ['/etc/xdg/autostart', '~/.config/autostart', '~/.config/openbox/autostart']


def check_startup_file():
    return any(
        os.path.isfile(dirname+'/'+DesktopEntryName)
        for dirname in map(os.path.expanduser, StartupPath))


def delete_startup_linux():
    try:
        for dirname in map(os.path.expanduser, StartupPath):
            fileName = dirname+'/'+DesktopEntryName
            if os.path.isfile(fileName):
                os.remove(fileName)
    except Exception, e:
        logger.debug("Delete startup fail:" + str(e))
        return False
    return True


def command_exist(command='gksu'):
    command += ' --help'
    try:
        if commands.getstatusoutput(command)[0] != 32512:
            logger.debug(command + " exist")
            return True
        else:
            logger.debug(command + " dosen't exist")
            return False
    except Exception:
        logger.debug(command + " dosen't exist")
        return False


def add_startup_linux():
    filename = os.path.abspath(__file__)
    dirname = os.path.dirname(filename)
    command = 'gksu' if command_exist() else 'sudo'

    if os.path.exists("/usr/share/applications/Tickeys.desktop"):
        # install by deb
        with open("/usr/share/applications/Tickeys.desktop") as f:
            DESKTOP_FILE = f.read()
    elif dirname.find('library.zip') != -1:
        realPath = dirname.strip('library.zip')
        # In a package
        DESKTOP_FILE = '''\
[Desktop Entry]
Type=Application
Categories=Application;
Path=%s
Exec=%s sh %s
Terminal=true
Icon=%s/tickeys.png
Hidden=false
NoDisplay=false
StartupNotify=true
X-GNOME-Autostart-enabled=true
Name=Tickeys
Comment=Instant audio feedback when typing. For Linux.
''' % (realPath, command, executable_filename, realPath)

    elif command_exist('tickeys'):
        # used pip installed
        DESKTOP_FILE = '''\
[Desktop Entry]
Type=Application
Categories=Application;
Exec=%s tickeys
Icon=%s/tickeys.png
Terminal=true
Hidden=false
NoDisplay=false
StartupNotify=true
X-GNOME-Autostart-enabled=true
Name=Tickeys
Comment=Instant audio feedback when typing. For Linux.
''' % (command, dirname)
    else:
        # not install yet(run in py)
        DESKTOP_FILE = '''\
[Desktop Entry]
Type=Application
Categories=Application;
Path=%s
Exec=%s python run.py
Terminal=true
Icon=%s/tickeys.png
Hidden=false
NoDisplay=false
StartupNotify=true
X-GNOME-Autostart-enabled=true
Name=Tickeys
Comment=Instant audio feedback when typing. For Linux.
''' % (dirname, command, dirname)

    try:
        for dirname in map(os.path.expanduser, StartupPath):
            if not os.path.exists(dirname):
                os.makedirs(dirname)

            if os.path.isdir(dirname):
                filename = os.path.join(dirname, DesktopEntryName)
                with open(filename, 'w') as fp:
                    fp.write(DESKTOP_FILE)
                os.chmod(filename, 0777)
    except Exception, e:
        logger.debug("Add to startup fail:" + str(e))
        return False
    return True



#!/usr/bin/env python
# coding: utf-8
import logging

# 创建一个logger
logger = logging.getLogger("logger")
logger.setLevel(logging.DEBUG)

# 创建一个handler，用于输出到控制台
sh = logging.StreamHandler()
sh.setLevel(logging.INFO)

# 创建一个handler，用于写入日志文件
fh = logging.FileHandler("/tmp/tickeys.log")
fh.setLevel(logging.DEBUG)

# 定义handler的输出格式
# fhfmt = logging.Formatter(
#     '[%(levelname)s] %(asctime)s %(filename)s[line:%(lineno)d] %(message)s')
fhfmt = logging.Formatter(
    '%(message)s  -%(filename)s[line:%(lineno)d]')
shfmt = logging.Formatter(
    '%(message)s')
fh.setFormatter(fhfmt)
sh.setFormatter(shfmt)

logger.addHandler(sh)
logger.addHandler(fh)
logger.debug('test')



#!/usr/bin/env python
# coding: utf-8
from logger import logger
import sys
import os

from windowManager import check_tickeys_running_status

__author__ = 'Huang xiongbiao(billo@qq.com)'


def run_GUI():
    check_root()
    check_system()
    try:
        from GUI import TickeysApp
        TickeysApp().run()
    except Exception, e:
        logger.exception(e)
        logger.error("Run GUI Fail, reason:")
        os._exit(0)


def check_root():
    logger.info("Root checking...")
    if os.getegid() != 0:
        logger.info("This program must be run as root..")
        sys.exit(0)
    logger.info("Root checking success. You have the root permission")
    logger.debug("File path:" + os.path.dirname(__file__))

def check_system():
    systems = ['Linux', 'SunOS', 'FreeBSD', 'Unix', 'OpenBSD', 'NetBSD']
    try:
        logger.info("System checking...")
        import platform
        system_name = platform.system()
        if system_name not in systems:
            logger.error("System %s is not supported." % system_name)
            sys.exit(0)
        else:
            logger.info("System checking success. Your system is supported")
    except Exception, e:
        logger.error("System checking fail:")
        logger.exception(e)
        sys.exit(0)


def main():
    logger.debug("Tickeys start........")
    is_running = check_tickeys_running_status()
    if is_running:
        return
    run_GUI()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# coding:utf-8

import os
import webbrowser
from windowManager import show_GUI

import pygtk
pygtk.require('2.0')

import gtk
gtk.gdk.threads_init()

try:
    import platform
    import appindicator
except:
    platform = None
    appindicator = None


class GtkTray():
    def __init__(self):
        logo_filename = 'tickeys.png'

        if platform and appindicator and platform.dist()[0].lower() == 'ubuntu':
            self.trayicon = self.ubuntu_trayicon(logo_filename)
        else:
            self.trayicon = self.gtk_trayicon(logo_filename)

    def ubuntu_trayicon(self, logo_filename):
        trayicon = appindicator.Indicator('Tickeys', 'indicator-messages', appindicator.CATEGORY_APPLICATION_STATUS)
        trayicon.set_status(appindicator.STATUS_ACTIVE)
        trayicon.set_attention_icon('indicator-messages-new')
        trayicon.set_icon(logo_filename)
        trayicon.set_menu(self.make_menu())

        return trayicon

    def gtk_trayicon(self, logo_filename):
        trayicon = gtk.StatusIcon()
        trayicon.set_from_file(logo_filename)

        trayicon.connect('popup-menu', lambda i, b, t: self.make_menu().popup(None, None, gtk.status_icon_position_menu, b, t, self.trayicon))
        trayicon.connect('activate', self.on_show)
        trayicon.set_tooltip('Tickeys')
        trayicon.set_visible(True)

        return trayicon

    def make_menu(self):
        menu = gtk.Menu()
        itemlist = [(u'显示界面', self.on_show),
                    (u'项目Github页', self.show_github_page),
                    (u'退出', self.on_quit)]
        for text, callback in itemlist:
            item = gtk.MenuItem(text)
            item.connect('activate', callback)
            item.show()
            menu.append(item)
        menu.show()
        return menu

    def on_show(self, widget=None, data=None):
        print "show the UI"
        show_GUI()

    def on_quit(self, widget, data=None):
        # module_init.stop_all()
        os._exit(0)
        gtk.main_quit()

    def show_github_page(self, widget=None, data=None):
        webbrowser.open_new("https://github.com/BillBillBillBill/Tickeys-linux")

    def serve_forever(self):
        gtk.gdk.threads_enter()
        gtk.main()
        gtk.gdk.threads_leave()


def main():
    sys_tray = GtkTray()
    sys_tray.serve_forever()

if __name__ == '__main__':
    main()



import ConfigParser
import os
from logger import logger


class Configer():
    """docstring for Configer"""
    def __init__(self, *arg):
        try:
            os.chdir(os.path.dirname(__file__))
        except Exception:
            pass
        self.cf = ConfigParser.ConfigParser()
        self.read_config()

    def init_config(self):
        self.style = 'mechanical'
        self.volume = 1.0
        self.pitch = 1.0
        self.save_config()

    def read_config(self):
        try:
            config_path = [
                "/usr/share/Tickeys",
                "/usr/share/Tickeys/config",
                "/usr/share/Tickeys/config/tickeys.conf"
            ]
            if not all([os.path.exists(cp) for cp in config_path]):
                if not os.path.exists("/usr/share/Tickeys"):
                    os.mkdir("/usr/share/Tickeys")
                if not os.path.exists("/usr/share/Tickeys/config"):
                    os.mkdir("/usr/share/Tickeys/config")
                self.init_config()
            else:
                self.cf.read('/usr/share/Tickeys/config/tickeys.conf')
                self.volume = self.cf.getfloat('options', 'volume')
                self.pitch = self.cf.getfloat('options', 'pitch')
                self.style = self.cf.get('options', 'style')
        except Exception, e:
            logger.debug(e)

    def save_config(self):
        if not self.cf.sections():
            self.cf.add_section('options')
        self.cf.set('options', 'volume', self.volume)
        self.cf.set('options', 'pitch', self.pitch)
        self.cf.set('options', 'style', self.style)

        with open('/usr/share/Tickeys/config/tickeys.conf', 'w') as f:
            self.cf.write(f)

    @property
    def volume(self):
        return self.volume

    @property
    def pitch(self):
        return self.pitch

    @property
    def style(self):
        return self.style



#!/usr/bin/env python
# coding: utf-8
from pygame import mixer
from logger import logger
from config import Configer
import json
import os

__author__ = 'Huang xiongbiao(billo@qq.com)'


class SoundPlayer():

    def __init__(self):
        try:
            os.chdir(os.path.dirname(__file__))
        except Exception:
            pass
        self.mixer = mixer
        self.mixer.init(frequency=22050)
        self.schemes = dict(
            (i['name'], i)
            for i in json.load(file('./Resources/data/schemes.json')))
        self.Configer = Configer()
        self.soundFileList = self.schemes[self.Configer.style]['files']
        self.key_audio_map = self.schemes[self.Configer.style]['key_audio_map']
        self.non_unique_count = self.schemes[self.Configer.style]['non_unique_count']
        self.set_pitch(self.Configer.pitch)
        self.changing = False  # avoiding play sound while caching
        self.soundEffectCache = []
        self.cache_sound_effect()

    # preload sound effect
    def cache_sound_effect(self):
        try:
            self.changing = True
            self.mixer.quit()
            self.mixer.init(frequency=int(self.adjustPitch*22050))
            self.soundEffectCache = []
            for effectFile in self.soundFileList:
                soundFile = './Resources/data/%s/%s' % \
                    (self.Configer.style, effectFile)
                logger.debug('Load sound file:' + soundFile)
                self.soundEffectCache.append(self.mixer.Sound(soundFile))
            self.set_volume(self.Configer.volume)

        except Exception, e:
            logger.error('Load sound files fail:' + str(e))
        finally:
            self.changing = False

    def save_config(func):
        def _deco(*arg):
            ret = func(*arg)
            arg[0].Configer.save_config()
            return ret
        return _deco

    @save_config
    def set_style(self, style):
        try:
            if self.Configer.style == style:
                return
            self.Configer.style = style
            self.soundFileList = self.schemes[self.Configer.style]['files']
            self.key_audio_map = self.schemes[self.Configer.style]['key_audio_map']
            self.non_unique_count = self.schemes[self.Configer.style]['non_unique_count']
            self.cache_sound_effect()
        except Exception, e:
            logger.error(e)

    @save_config
    def set_volume(self, volume):
        self.Configer.volume = volume
        for soundEffect in self.soundEffectCache:
            soundEffect.set_volume(self.Configer.volume)

    # May raise nonfluency when init the mixer
    @save_config
    def set_pitch(self, pitch):
        # if abs(self.mixer.get_init()[0]/22050.0 - pitch) < 0.1:
        #     return
        # it's weird
        self.Configer.pitch = pitch
        self.adjustPitch = 0.65 + 0.3 * pitch if pitch < 1 else pitch + 0.9
        self.cache_sound_effect()

    def get_infor(self):
        return {
            'style': self.Configer.style,
            'volume': self.Configer.volume,
            'pitch': self.Configer.pitch,
        }

    def play(self, key):

        if not self.key_audio_map.get(str(key)):
            self.key_audio_map[str(key)] = key % self.non_unique_count

        # avoid the situation that play sound while caching
        while self.changing:
            continue

        self.soundEffectCache[self.key_audio_map[str(key)]].play()
        # dir(self.soundEffectCache[self.key_audio_map[str(key)]])



#!/usr/bin/env python
# coding: utf-8
import os
import re
import threading
from evdev import InputDevice
from select import select
from soundPlayer import SoundPlayer
from logger import logger

from windowManager import show_GUI

__author__ = 'Huang xiongbiao(billo@qq.com)'

# input device file path
deviceFilePath = '/sys/class/input/'


class KeyboardHandler():

    def __init__(self):
        logger.debug("Keyboard deteccter created.")
        self.detect_keyboard_list = self.find_keyboard()
        self.threads = []
        self.input_record_list = []
        self.hot_key_list = [16, 30, 44, 2, 3, 4]      # QAZ123
        self.hot_key_list2 = [16, 30, 44, 79, 80, 81]  # QAZ123 with 123 in side keyboard
        self.sp = SoundPlayer()
        self.show_device()

    # list all event's name and its device and record it
    def show_device(self):
        # os.chdir(deviceFilePath)
        logger.debug("List all device")
        for i in os.listdir(deviceFilePath):
            namePath = deviceFilePath + i + '/device/name'
            if os.path.isfile(namePath):
                logger.debug("Name: %s Device: %s" % (i, file(namePath).read()))

    def set_style(self, style):
        self.sp.set_style(style)

    def set_volume(self, volume):
        self.sp.set_volume(volume)

    def set_pitch(self, pitch):
        self.sp.set_pitch(pitch)

    def get_player_infor(self):
        return self.sp.get_infor()

    # return with a list of keyboard's event
    def find_keyboard(self):
        keyboard_list = []

        device_info = open('/proc/bus/input/devices').read().lower().split('\n\n')
        logger.debug("/proc/bus/input/devices:%s" % device_info)
        exclude_pattern = 'touch|web|cam|hdmi|button|mic|phone|speak'
        for i in filter(lambda i: not re.search(exclude_pattern, i) and
            ((not re.search('mouse', i) and re.search('bus=0003', i)) or re.search('keyboard', i))
            , device_info):
            m = re.search('event\d+', i)
            if m:
                keyboard_list.append(m.group())
        assert len(keyboard_list) > 0
        logger.debug("Keyboard list: %s" % keyboard_list)
        return keyboard_list

    # event.value:1 for pressed, 0 for release
    def detect_input_from_event(self, eventName):
        dev = InputDevice('/dev/input/' + eventName)
        while True:
            select([dev], [], [])
            for event in dev.read():
                if (event.value == 1 or event.value == 0) and event.code != 0:
                    if event.value == 1:
                        self.sp.play(event.code)
                        self.check_show_window(event.code)
                    logger.debug(
                        "Key: %s Status: %s" %
                        (event.code, "pressed" if event.value else "release"))

    # check input if satisfy the hotkey
    def check_show_window(self, key_code):
        if self.input_record_list and key_code == self.input_record_list[-1]:
            return
        input_record_length = len(self.input_record_list)
        next_key_code = self.hot_key_list[input_record_length]
        next_key_code2 = self.hot_key_list2[input_record_length]
        if key_code == next_key_code or key_code == next_key_code2:
            self.input_record_list.append(key_code)
            if input_record_length == 5:
                show_GUI()
                self.input_record_list = []
        else:
            # clear the record if not satisfy
            self.input_record_list = []

    def start_detecting(self):

        for i in self.detect_keyboard_list:
            t = threading.Thread(target=self.detect_input_from_event, args=(i,))
            self.threads.append(t)
            t.start()

    # kill all threads
    def stop_detecting(self):
        for t in self.threads:
            t._Thread__stop()

if __name__ == '__main__':
    detecter = KeyboardHandler()
    detecter.start_detecting()



import shutil
import os
from cx_Freeze import setup, Executable
from __init__ import __version__

executable_filename = "tickeys"

if __name__ == '__main__':

    buildOptions = dict(
        compressed=True,
        include_files=['kivy', 'tickeys', 'Resources'],)

    setup(
        name=executable_filename,
        version=__version__,
        description='Instant audio feedback when typing. For Linux.',
        options=dict(build_exe=buildOptions),
        executables=[Executable("run.py")])

    # print "Move resources file..."
    # shutil.copytree("../Resources", os.getcwd() + "/build/Resources")

    print "Add readme file..."
    shutil.copyfile("../README.md", os.getcwd() + "/build/README.md")

    print "Add some docs..."
    shutil.copyfile("../AUTHOURS", os.getcwd() + "/build/AUTHOURS")
    shutil.copyfile("../Changelog", os.getcwd() + "/build/Changelog")
    shutil.copyfile("../LICENSE", os.getcwd() + "/build/LICENSE")


from setuptools import setup, find_packages
from tickeys import __version__, __author__, __email__

requirements = [
    'cython==0.20.2',
    'kivy==1.9.0',
    'evdev',
    'notify2',
]


setup(name='tickeys',
      version=__version__,
      download_url='git@github.com:BillBillBillBill/Tickeys-linux.git',
      packages=['tickeys'],
      package_dir={'tickeys': 'tickeys'},
      include_package_data=True,
      package_data={'tickeys': ['*.*']},
      # data_files=[],
      classifiers=[
          'Intended Audience :: End Users/Desktop',
          'Development Status :: 3 - Alpha',
          'Environment :: X11 Applications',
          'License :: OSI Approved :: MIT License',
          'Operating System :: POSIX :: Linux',
          'Natural Language :: Chinese (Simplified)',
          'Programming Language :: Python :: 2.7',
      ],
      author=__author__,
      author_email=__email__,
      description='Instant audio feedback when typing. For Linux.',
      long_description=open('README.md').read(),
      keywords='keyboard typing audio feedback ',
      url='https://github.com/BillBillBillBill/Tickeys-linux',
      license='MIT',
      entry_points={
          'console_scripts': ['tickeys = tickeys:main']
      },
      install_requires=requirements,
      tests_require=requirements)



# coding: utf-8
import commands

from logger import logger
from threading import Thread

def save_terminal_window_id():
    try:
        stat, terminalId = commands.getstatusoutput('xdotool getactivewindow')
        with open("/tmp/tickeys_terminal_window_id", "w+") as f:
            if stat == 0:
                f.write(terminalId)
            else:
                f.write(0)
    except Exception, e:
        logger.error("Save terminal window id fail:" + str(e))


def read_terminal_window_id():
    with open("/tmp/tickeys_terminal_window_id", "r") as f:
        return f.read()


def hide_terminal():
    try:
        terminalId = read_terminal_window_id()
        if not terminalId:
            return
        commands.getstatusoutput(
            "xdotool windowactivate --sync %s" % terminalId)
        commands.getstatusoutput(
            "xdotool getactivewindow windowunmap")
    except Exception,e:
        logger.error(str(e))


def save_GUI_window_id():
    try:
        stat, GUIID = commands.getstatusoutput('xdotool getactivewindow')
        with open("/tmp/tickeys_GUI_window_id", "w+") as f:
            if stat == 0:
                f.write(GUIID)
            else:
                f.write(0)
    except Exception, e:
        logger.error("Save GUI window id fail:" + str(e))


def read_GUI_window_id():
    with open("/tmp/tickeys_GUI_window_id", "r") as f:
        return f.read()


def hide_GUI():
    try:
        GUIID = read_GUI_window_id()
        commands.getstatusoutput(
                'xdotool windowunmap --sync %s' % GUIID)
    except Exception,e:
        logger.error(str(e))


def show_GUI():
    def show_notify():
        try:
            import notify2
            notify2.init('Tickeys')
            title = 'Tickeys'
            body = '程序“xdotool”尚未安装, 无法隐藏窗口。'
            iconfile = os.getcwd() + '/tickeys.png'
            notify = notify2.Notification(title, body, iconfile)
            notify.show()
        except Exception:
            return
    try:
        GUIID = read_GUI_window_id()
        if not GUIID or GUIID == "0":
            Thread(target=show_notify).start()
            return
        else:
            # read window ids
            command = "xdotool windowmap --sync %s && xdotool windowactivate --sync %s" % (GUIID, GUIID)
            stat, output = commands.getstatusoutput(command)
            return str(stat)
    except Exception, e:
        logger.error(str(e))
        return '256'

def check_tickeys_running_status():
    save_terminal_window_id()
    stat = show_GUI()
    if stat != "0":
        return False
    else:
        print "Tickeys is already running, show it"
        return True



#!/usr/bin/env python
# coding: utf-8
from kivy.app import App
from kivy.uix.spinner import Spinner
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.gridlayout import GridLayout
from keyboardHandler import KeyboardHandler
from kivy.lang import Builder
from startupHandler import add_startup_linux, check_startup_file, delete_startup_linux
from logger import logger
from config import Configer
from __init__ import __version__, debug_mode

import sys
import os

from threading import Thread

from windowManager import hide_GUI, save_GUI_window_id

reload(sys)
sys.setdefaultencoding("utf-8")

# set font
from kivy.core.text import Label
try:
    os.chdir(os.path.dirname(__file__))
except Exception:
    pass
Label.register("DroidSans", "./Resources/fonts/DroidSansFallbackFull.ttf")

Builder.load_string('''
<Main>:
    pos: 0,0
    padding: 50
    rows: 6
    row_force_default: True
    row_default_height: 50
    spacing: 50
    orientation: 'vertical'

    canvas:
        Color:
            rgba: 0.368, 0.384, 0.447, 0.8
        Rectangle:
            pos: 0,0
            size: self.size
    Label:
        bold: True
        text: 'Tickeys'
        font_size: 50
        size_hint_y: None

    SpinnerRow
    AdjustVol
    AdjustPitch
    ExitAndSwitchRow
    InforRow


<AdjustVol>
    Label:
        bold: True
        color: 1, 1, 1, 1
        font_size: 33
        size_hint_x: None
        width: 250
        text: '音量：'
    Slider:
        min: 0.0
        max: 1.0
        value: root.parent.Configer.volume
        width: 300
        on_value: root.setVolume(self.value)

<AdjustPitch>
    Label:
        bold: True
        color: 1, 1, 1, 1
        font_size: 33
        size_hint_x: None
        width: 250
        text: '音调：'
    Slider:
        min: 0.0
        max: 3.0
        value: root.parent.Configer.pitch
        width: 300
        on_value: root.setPitch(self.value)


<SpinnerRow>:
    Label:
        bold: True
        color: 1, 1, 1, 1
        font_size: 33
        size_hint_x: None
        text: "声音方案："
        width: 250
    EffectSpinner:
        on_text: root.change_style()


<EffectSpinner>:
    bold: True
    font_size: 30
    text: root.get_style_name()
    background_color: 3, 3, 3, 1
    color: 0.1, 0.67, 0.93, 1
    values:['冒泡', '打字机', '机械键盘', '剑气', 'Cherry G80-3000', 'Cherry G80-3494', '爆裂鼓手']

<ExitAndSwitchRow>:
    Label:
        size_hint_x: None
        width: root.width/6.0 - 120
    Label:
        size_hint_x: None
        color: 1, 1, 1, 1
        font_size: 17
        width: root.width/6.0 + 60
        text: '开机时自启动：'
    Switch:
        size_hint_x: None
        width: 40
        id: switcher
        active: root.have_added
        on_active: root.add_delete_startup_file(self.active)
    Label:
        size_hint_x: None
        width: root.width/6.0 - 35
    Button:
        size_hint_x: None
        width: 150
        font_size: 20
        background_color: 3, 3, 3, 1
        bold: True
        text: "退出"
        color: 0,0,0,1
        on_press: root.Exit()
    Label:
        size_hint_x: None
        width: 20
    Button:
        size_hint_x: None
        width: 150
        font_size: 20
        background_color: 3, 3, 3, 1
        bold: True
        text: "隐藏"
        color: 0,0,0,1
        on_press: root.Hide()


<InforRow>:
    Label:
        color: 0.8, 0.8, 0.8, 1
        font_size: 20
        size_hint_x: None
        text: root.get_version()
        width: root.width/3.0
    Label:
        color: 0.8, 0.8, 0.8, 1
        font_size: 20
        size_hint_x: None
        markup: True
        text: "[ref=项目主页]项目主页[/ref]"
        width: root.width/3.0
        on_ref_press:root.open_project_website()
    Label:
        color: 0.8, 0.8, 0.8, 1
        font_size: 20
        size_hint_x: None
        text: "作者： Bill (billo@qq.com)"
        width: root.width/3.0
        border: 1,1,1,1
        on_touch_move:
'''.encode('utf-8'))


def show_notify(notify_content=""):
    try:
        import notify2
        notify2.init('Tickeys')
        title = 'Tickeys'
        icon_file_path = os.getcwd() + '/tickeys.png'
        notify = notify2.Notification(title, notify_content, icon_file_path)
        notify.show()
    except Exception, e:
        logger.exception(e)
        logger.error("show notify fail")


def show_startup_notify():
    notify_content = '<span style="color: #00B8CB; font-size:15px">Tickeys</span>正在运行\n随时按<span style="color: #00B8CB">QAZ123</span>唤出设置窗口'
    show_notify(notify_content)


def check_update_and_notify():
    try:
        import urllib
        import json
        logger.info("Version checking...")
        r = urllib.urlopen('http://billbill.sinaapp.com/tickeys')
        return_msg = json.loads(r.read())
        if return_msg["version"] <= __version__:
            logger.debug("Version checking success. It is the latest version...")
        else:
            # show update notify
            notify_content = '<span style="color: #00B8CB; font-size:15px">Tickeys</span>有可用的<span style="color: #FF4500">更新：</span>\n 版本：%s \n 内容：%s' % (return_msg["version"], return_msg["update"])
            show_notify(notify_content)
    except Exception, e:
        logger.exception(e)
        logger.error("Version checking fail:" + str(e))


class EffectSpinner(Spinner):
    def get_style_name(self):
        style_display_name_map = {
            'bubble': "冒泡",
            'typewriter': "打字机",
            'mechanical': "机械键盘",
            'sword': "剑气",
            'Cherry_G80_3000': "Cherry G80-3000",
            'Cherry_G80_3494': "Cherry G80-3494",
            'drum': "爆裂鼓手"
        }
        name = self.parent.parent.Configer.style
        if name not in style_display_name_map:
            display_name = name
        else:
            display_name = style_display_name_map[name]
        return display_name


class SpinnerRow(BoxLayout):
    def change_style(self):
        style_display_name_map = {
            '冒泡': "bubble",
            '打字机': "typewriter",
            '机械键盘': "mechanical",
            '剑气': "sword",
            'Cherry G80-3000': "Cherry_G80_3000",
            'Cherry G80-3494': "Cherry_G80_3494",
            '爆裂鼓手': "drum"
        }
        # for safe
        if self.children[0].text not in style_display_name_map:
            return
        style_name = style_display_name_map[self.children[0].text]
        self.parent.detecter.set_style(style_name)


class AdjustVol(BoxLayout):
    def setVolume(self, volume):
        self.parent.detecter.set_volume(volume)


class AdjustPitch(BoxLayout):
    def setPitch(self, pitch):
        self.parent.detecter.set_pitch(pitch)


class SwitcherRow(BoxLayout):
    pass


class ExitAndSwitchRow(BoxLayout):
    def Exit(self):
        os._exit(0)

    def Hide(self):
        self.parent.Hide()

    def add_delete_startup_file(self, active):
        if active:
            add_startup_linux()
        else:
            delete_startup_linux()

    @property
    def have_added(self):
        return check_startup_file()


class InforRow(BoxLayout):
    def open_project_website(self, *args):
        from webbrowser import open_new
        open_new("https://github.com/BillBillBillBill/Tickeys-linux")

    def get_version(self):
        return 'Version： ' + __version__


class Main(GridLayout):
    def __init__(self, *args, **kwargs):
        self.Configer = Configer()
        super(Main, self).__init__(**kwargs)
        save_GUI_window_id()
        self.Hide()
        # set up keyboard detecter
        self.detecter = KeyboardHandler()
        self.detecter.start_detecting()
        # show notify message
        Thread(target=show_startup_notify).start()
        Thread(target=check_update_and_notify).start()

    def Hide(self):
        if not debug_mode:
            hide_GUI()


class TickeysApp(App):
    def __init__(self, *args, **kwargs):
        super(TickeysApp, self).__init__(**kwargs)

    def build(self):
        self.icon = 'tickeys.png'
        root = Main()
        return root

    def on_stop(self):
        os._exit(0)


if __name__ == '__main__':
    TickeysApp().run()
